==> multi-tenant-runtime-1.7.0-sources.jar: com/sap/cloud/mt/runtime/DataSourceCreator.java <==
/******************************************************************************
 * © 2020 SAP SE or an SAP affiliate company. All rights reserved.            *
 ******************************************************************************/
package com.sap.cloud.mt.runtime;

import java.lang.reflect.InvocationTargetException;
import java.lang.reflect.Method;
import java.util.stream.Stream;

import javax.sql.DataSource;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import com.sap.cloud.mt.runtime.DataPoolSettings.Parameter;
import com.sap.cloud.mt.subscription.DataSourceInfo;
import com.sap.cloud.mt.subscription.exceptions.InternalError;

public abstract class DataSourceCreator {
    private static final Logger logger= LoggerFactory.getLogger(DataSourceCreator.class);

    public final DataSource create(DataSourceInfo info, EnvironmentAccess env) throws InternalError{
        DataSource dataSource = build(info);
        setPoolParameters(dataSource, env);
        return dataSource;
    }

    private final void setPoolParameters(DataSource dataSource, EnvironmentAccess env) {
        getPoolParameters().forEach(p -> {
            Object para = env.getProperty(p.getNameInEnv(), p.getType());
            if (para != null) {
                try {
                    getMethod(dataSource, p).invoke(dataSource, para);
                } catch (NoSuchMethodException | IllegalAccessException | InvocationTargetException e) {
                    throw new RuntimeException(e);
                }
            }
        });
    }

    protected abstract DataSource build(DataSourceInfo info) throws InternalError;

    protected abstract Stream<Parameter> getPoolParameters();

    private Method getMethod(DataSource dataSource, DataPoolSettings.Parameter p) throws NoSuchMethodException {
        try {
            return dataSource.getClass().getMethod(p.getSetterName(), p.getType());
        } catch (NoSuchMethodException e) {
            logger.debug("Try with another type");
            if (p.getType() == boolean.class) {
                return dataSource.getClass().getMethod(p.getSetterName(), Boolean.class);
            } else if (p.getType() == int.class) {
                return dataSource.getClass().getMethod(p.getSetterName(), Integer.class);
            } else if (p.getType() == long.class) {
                return dataSource.getClass().getMethod(p.getSetterName(), Long.class);
            } else throw new NoSuchMethodException();
        }
    }
}

==> multi-tenant-runtime-2.4.1-sources.jar: com/sap/cloud/mt/runtime/DataSourceCreator.java <==
/******************************************************************************
 * © 2020 SAP SE or an SAP affiliate company. All rights reserved.            *
 ******************************************************************************/
package com.sap.cloud.mt.runtime;

import com.sap.cloud.mt.runtime.DataPoolSettings.Parameter;
import com.sap.cloud.mt.subscription.DataSourceInfo;
import com.sap.cloud.mt.subscription.exceptions.InternalError;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import javax.sql.DataSource;
import java.lang.reflect.InvocationTargetException;
import java.lang.reflect.Method;
import java.util.Map;
import java.util.Properties;
import java.util.stream.Stream;

public abstract class DataSourceCreator {
    private static final Logger logger = LoggerFactory.getLogger(DataSourceCreator.class);

    public final DataSource create(DataSourceInfo info, EnvironmentAccess env) throws InternalError {
        DataSource dataSource = build(info, env);
        setPoolParameters(dataSource, env);
        return dataSource;
    }

    private final void setPoolParameters(DataSource dataSource, EnvironmentAccess env) {
        getPoolParameters().forEach(p -> {
            Object para;
            if (p.getType() != Properties.class) {
                para = env.getProperty(p.getNormalizedNameInEnv(), p.getType());
            } else {
                para = env.getPropertiesForPrefix(p.getNormalizedNameInEnv());
            }
            if (para != null) {
                try {
                    if (p.getType() != Properties.class) {
                        getMethod(dataSource, p).invoke(dataSource, para);
                    } else {
                        Properties properties = new Properties();
                        properties.putAll((Map<Object, Object>) para);
                        getMethod(dataSource, p).invoke(dataSource, properties);
                    }
                } catch (NoSuchMethodException | IllegalAccessException | InvocationTargetException e) {
                    throw new RuntimeException(e);
                }
            }
        });
    }

    protected abstract DataSource build(DataSourceInfo info, EnvironmentAccess env) throws InternalError;

    protected abstract Stream<Parameter> getPoolParameters();

    private Method getMethod(DataSource dataSource, DataPoolSettings.Parameter p) throws NoSuchMethodException {
        try {
            return dataSource.getClass().getMethod(p.getSetterName(), p.getType());
        } catch (NoSuchMethodException e) {
            logger.debug("Try with another type");
            if (p.getType() == boolean.class) {
                return dataSource.getClass().getMethod(p.getSetterName(), Boolean.class);
            } else if (p.getType() == int.class) {
                return dataSource.getClass().getMethod(p.getSetterName(), Integer.class);
            } else if (p.getType() == long.class) {
                return dataSource.getClass().getMethod(p.getSetterName(), Long.class);
            } else throw new NoSuchMethodException();
        }
    }
}
